@@ -6,8 +6,14 @@ Author: Gerd Duscher
 """
 
 import numpy as np
+Qt_available = True
+try:
+    from PyQt5 import QtCore, QtGui, QtWidgets
+
+except:
+    Qt_available = False
+    print('Qt dialogs are not available')
 
-from PyQt5 import QtWidgets, QtCore, QtGui
 import sidpy
 import matplotlib.patches as patches
 from matplotlib.widgets import RectangleSelector, SpanSelector
@@ -36,317 +42,318 @@ CurveVisualizer = eels_dialog.CurveVisualizer
 InfoDialog = info_dialog.InfoDialog
 PeakFitDialog = peak_dialog.PeakFitDialog
 
+if Qt_available:
 
-class PeriodicTableDialog(QtWidgets.QDialog):
-    """ Modal dialog to get a selection of elements.
+    class PeriodicTableDialog(QtWidgets.QDialog):
+        """ Modal dialog to get a selection of elements.
 
-    Elements that are not having a valid cross-sections are disabled.
+        Elements that are not having a valid cross-sections are disabled.
 
-    Parameters
-    ----------
-    initial_elements: list of str
-        the elements that are already selected
-    energy_scale: list or numpy array
-        energy-scale of spectrum/spectra to determine likely edges
+        Parameters
+        ----------
+        initial_elements: list of str
+            the elements that are already selected
+        energy_scale: list or numpy array
+            energy-scale of spectrum/spectra to determine likely edges
 
-    Returns
-    -------
-    list of strings: elements.
+        Returns
+        -------
+        list of strings: elements.
 
-    Example
-    -------
-    >> PT_dialog =  periodic_table_dialog(None, ['Mn', 'O'])
-    >> if PT_dialog.exec_() == periodic_table_dialog.Accepted:
-    >>     selected_elements = PT_dialog.get_output()
-    >> print(selected_elements)
-    """
+        Example
+        -------
+        >> PT_dialog =  periodic_table_dialog(None, ['Mn', 'O'])
+        >> if PT_dialog.exec_() == periodic_table_dialog.Accepted:
+        >>     selected_elements = PT_dialog.get_output()
+        >> print(selected_elements)
+        """
 
-    signal_selected = QtCore.pyqtSignal(list)
+        signal_selected = QtCore.pyqtSignal(list)
 
-    def __init__(self, initial_elements=None, energy_scale=None, parent=None):
-        super(PeriodicTableDialog, self).__init__(None, QtCore.Qt.WindowStaysOnTopHint)
+        def __init__(self, initial_elements=None, energy_scale=None, parent=None):
+            super(PeriodicTableDialog, self).__init__(None, QtCore.Qt.WindowStaysOnTopHint)
 
-        if initial_elements is None:
-            initial_elements = [' ']
-        if energy_scale is None:
-            energy_scale = [100., 150., 200.]
-        self.parent = parent
-        self._output = []
-        self.elements_selected = initial_elements
-        self.energy_scale = np.array(energy_scale)
+            if initial_elements is None:
+                initial_elements = [' ']
+            if energy_scale is None:
+                energy_scale = [100., 150., 200.]
+            self.parent = parent
+            self._output = []
+            self.elements_selected = initial_elements
+            self.energy_scale = np.array(energy_scale)
 
-        self.setWindowTitle("Periodic Table")
-        likely_edges = get_likely_edges(self.energy_scale)
-        self.likely_edges = likely_edges
+            self.setWindowTitle("Periodic Table")
+            likely_edges = get_likely_edges(self.energy_scale)
+            self.likely_edges = likely_edges
 
-        # GD:font = wx.Font(10, wx.MODERN, wx.NORMAL, wx.BOLD)
-        self.buttons1 = []
-        self.button = []
-        self.pt_info = get_periodic_table_info()
-        self.init_ui()
+            # GD:font = wx.Font(10, wx.MODERN, wx.NORMAL, wx.BOLD)
+            self.buttons1 = []
+            self.button = []
+            self.pt_info = get_periodic_table_info()
+            self.init_ui()
 
-        for button in self.button:
-            if button.text() in initial_elements:
-                button.toggle()
-            pass
+            for button in self.button:
+                if button.text() in initial_elements:
+                    button.toggle()
+                pass
 
-    def on_close(self):
-        self.get_output()
-        self.signal_selected[list].emit(self._output)
-        self.accept()
+        def on_close(self):
+            self.get_output()
+            self.signal_selected[list].emit(self._output)
+            self.accept()
 
-    def get_output(self):
-        self._output = []
-        for btn in self.button:
-            if btn.isChecked():
-                self._output.append(btn.text())
+        def get_output(self):
+            self._output = []
+            for btn in self.button:
+                if btn.isChecked():
+                    self._output.append(btn.text())
 
-    def exec_(self):
-        super(PeriodicTableDialog, self).exec_()
-        return self._output
+        def exec_(self):
+            super(PeriodicTableDialog, self).exec_()
+            return self._output
 
-    def init_ui(self):
+        def init_ui(self):
 
-        v_sizer = QtWidgets.QVBoxLayout()
-        g_sizer = QtWidgets.QGridLayout()
+            v_sizer = QtWidgets.QVBoxLayout()
+            g_sizer = QtWidgets.QGridLayout()
 
-        main_group = QtWidgets.QWidget()
+            main_group = QtWidgets.QWidget()
 
-        color1 = "background-color: lightblue;\n"
-        color1l = "background-color: dodgerblue;\n"
-        color2 = "background-color: coral;\n"
+            color1 = "background-color: lightblue;\n"
+            color1l = "background-color: dodgerblue;\n"
+            color2 = "background-color: coral;\n"
 
-        for symbol, parameter in self.pt_info.items():
-            self.button.append(QtWidgets.QPushButton(symbol))
-            if parameter['PT_row'] > 7:
-                self.button[-1].setStyleSheet(color2)
-            elif '*' in symbol:
-                self.button[-1].setStyleSheet(color2)
-            else:
-                if symbol in self.likely_edges:
-                    self.button[-1].setStyleSheet(color1l)
+            for symbol, parameter in self.pt_info.items():
+                self.button.append(QtWidgets.QPushButton(symbol))
+                if parameter['PT_row'] > 7:
+                    self.button[-1].setStyleSheet(color2)
+                elif '*' in symbol:
+                    self.button[-1].setStyleSheet(color2)
                 else:
-                    self.button[-1].setStyleSheet(color1)
-            if parameter['Z'] == 0:
-                self.button[-1].setEnabled(False)
-            self.button[-1].setFixedWidth(50)
-            self.button[-1].setCheckable(True)
-            g_sizer.addWidget(self.button[-1], parameter['PT_row'], parameter['PT_col'])
-        main_group.setLayout(g_sizer)
+                    if symbol in self.likely_edges:
+                        self.button[-1].setStyleSheet(color1l)
+                    else:
+                        self.button[-1].setStyleSheet(color1)
+                if parameter['Z'] == 0:
+                    self.button[-1].setEnabled(False)
+                self.button[-1].setFixedWidth(50)
+                self.button[-1].setCheckable(True)
+                g_sizer.addWidget(self.button[-1], parameter['PT_row'], parameter['PT_col'])
+            main_group.setLayout(g_sizer)
 
-        v_sizer.addWidget(main_group)
-        self.setLayout(v_sizer)
+            v_sizer.addWidget(main_group)
+            self.setLayout(v_sizer)
 
-        ok_button = QtWidgets.QPushButton('OK')
-        ok_button.clicked.connect(self.on_close)
+            ok_button = QtWidgets.QPushButton('OK')
+            ok_button.clicked.connect(self.on_close)
 
-        v_sizer.addWidget(ok_button)
-        self.setLayout(v_sizer)
+            v_sizer.addWidget(ok_button)
+            self.setLayout(v_sizer)
 
 
-class EnergySelector(QtWidgets.QDialog):
-    """Dialog and cursor to set energy scale"""
+    class EnergySelector(QtWidgets.QDialog):
+        """Dialog and cursor to set energy scale"""
 
-    signal_selected = QtCore.pyqtSignal(bool)
+        signal_selected = QtCore.pyqtSignal(bool)
 
-    def __init__(self, dset=None):
-        super(EnergySelector, self).__init__(None, QtCore.Qt.WindowStaysOnTopHint)
+        def __init__(self, dset=None):
+            super(EnergySelector, self).__init__(None, QtCore.Qt.WindowStaysOnTopHint)
 
-        if not isinstance(dset, sidpy.Dataset):
-            return
-        if dset is None:
-            return
-        if dset.view is None:
-            return
-        self.dataset = dset
-
-        if hasattr(dset.view, 'axis'):
-            self.axis = dset.view.axis
-            # self.setWindowTitle('p')
-        elif hasattr(dset.view, 'axes'):
-            self.axis = dset.view.axes[1]
-        else:
-            return
-
-        self.spec_dim = -1
-        for dim, axis in self.dataset._axes.items():
-            if axis.dimension_type == sidpy.DimensionType.SPECTRAL:
-                self.spec_dim = dim
-        if self.spec_dim < 0:
-            raise TypeError('We need at least one SPECTRAL dimension')
-
-        self.energy_scale = self.dataset._axes[self.spec_dim].values
-        self.dispersion = self.energy_scale[1] - self.energy_scale[0]
-        self.offset = self.energy_scale[0]
-        self.spectrum = np.zeros(2)
-
-        self.change = 0
-
-        self.x_min = self.energy_scale[int(len(self.energy_scale)/4)]
-        self.x_max = self.energy_scale[int(len(self.energy_scale) / 4*3)]
-        self.setWindowTitle("Select Energy")
-
-        valid_float = QtGui.QDoubleValidator()
-
-        layout = QtWidgets.QGridLayout()
-        layout.setVerticalSpacing(2)
-        self.label1 = QtWidgets.QLabel('Start:')
-        self.edit1 = QtWidgets.QLineEdit('0')
-        self.edit1.setValidator(valid_float)
-        self.unit1 = QtWidgets.QLabel('eV')
-
-        self.label2 = QtWidgets.QLabel('End:')
-        self.edit2 = QtWidgets.QLineEdit('0')
-        self.edit2.setValidator(valid_float)
-        self.unit2 = QtWidgets.QLabel('eV')
-
-        self.label3 = QtWidgets.QLabel('Dispersion:')
-        self.edit3 = QtWidgets.QLineEdit('0')
-        self.edit3.setValidator(valid_float)
-        self.unit3 = QtWidgets.QLabel('eV')
-
-        self.edit1.editingFinished.connect(self.on_enter)
-        self.edit2.editingFinished.connect(self.on_enter)
-        self.edit3.editingFinished.connect(self.on_enter)
-
-        layout.addWidget(self.label1, 0, 0)
-        layout.addWidget(self.edit1, 0, 1)
-        layout.addWidget(self.unit1, 0, 2)
-
-        layout.addWidget(self.label2, 1, 0)
-        layout.addWidget(self.edit2, 1, 1)
-        layout.addWidget(self.unit2, 1, 2)
-
-        layout.addWidget(self.label3, 2, 0)
-        layout.addWidget(self.edit3, 2, 1)
-        layout.addWidget(self.unit3, 2, 2)
-
-        self.ok_button = QtWidgets.QPushButton('OK')
-        self.ok_button.clicked.connect(self.on_close)
-        self.cancel_button = QtWidgets.QPushButton('Cancel')
-        self.cancel_button.clicked.connect(self.on_close)
-
-        layout.addWidget(self.ok_button, 3, 0)
-        layout.addWidget(self.cancel_button, 3, 2)
-
-        self.setLayout(layout)
-        self.edit1.setFocus()
-        self.plot()
-
-        self.selector = SpanSelector(self.axis, self.line_select_callback, 
-                                     direction="horizontal", 
-                                     span_stays=True,
-                                     props=dict(facecolor='blue', alpha=0.2))
-        self.edit1.setText(f'{self.x_min:.3f}')
-        self.edit2.setText(f'{self.x_max:.3f}')
-        self.edit3.setText(f'{self.dispersion:.4f}')
-        self.update()
-
-    def line_select_callback(self, eclick, erelease):
-        y_min, y_max = self.axis.get_ylim()
-        self.x_min = self.selector.extents[0]
-        self.x_max = self.selector.extents[1]
-        # self.selector.extents = (self.x_min, self.x_max, y_min, y_max)
-
-        self.edit1.setText(f'{self.x_min:.3f}')
-        self.edit2.setText(f'{self.x_max:.3f}')
-
-    def on_enter(self):
-        sender = self.sender()
-
-        if sender == self.edit1:
-            value = float(str(sender.displayText()).strip())
-            if value == self.x_min:
+            if not isinstance(dset, sidpy.Dataset):
                 return
-            self.change = value - self.x_min
-            self.x_min += self.change
-            self.x_max += self.change
-            self.offset += self.change
-
-            self.edit1.setText(f"{self.x_min:.2f}")
-            self.edit2.setText(f"{self.x_max:.2f}")
-
-            self.energy_scale = np.arange(len(self.energy_scale)) * self.dispersion + self.offset
-
-            self.update()
-            # self.axis.draw()
-            # self.setWindowTitle(f'shift, {self.change}, {self.x_min}')
-
-        elif sender == self.edit2:
-            value = float(str(sender.displayText()).strip())
-            if value == self.x_max:
+            if dset is None:
                 return
-            start_channel = np.searchsorted(self.energy_scale, self.x_min)
-            end_channel = np.searchsorted(self.energy_scale, self.x_max)
+            if dset.view is None:
+                return
+            self.dataset = dset
 
-            self.x_max = value
-
-            if end_channel - start_channel != 0:
-                self.dispersion = (self.x_max - self.x_min) / (end_channel - start_channel)
-            self.offset = self.x_min - start_channel * self.dispersion
-            self.edit2.setText(f"{self.x_max:.3f}")
-            self.edit3.setText(f"{self.dispersion:.4f}")
-            self.energy_scale = np.arange(len(self.energy_scale)) * self.dispersion + self.offset
-
-            self.update()
-            # self.axis.draw()
-            # self.setWindowTitle(f'range, {self.change}, {self.dispersion}')
-
-        elif sender == self.edit3:
-            value = float(str(sender.displayText()).strip())
-            if self.dispersion == value:
+            if hasattr(dset.view, 'axis'):
+                self.axis = dset.view.axis
+                # self.setWindowTitle('p')
+            elif hasattr(dset.view, 'axes'):
+                self.axis = dset.view.axes[1]
+            else:
                 return
 
-            start_channel = np.searchsorted(self.energy_scale, self.x_min)
-            end_channel = np.searchsorted(self.energy_scale, self.x_max)
-            self.dispersion = value
-            self.energy_scale = np.arange(len(self.energy_scale)) * self.dispersion + self.offset
-            self.x_min = self.energy_scale[start_channel]
-            self.x_max = self.energy_scale[end_channel]
-            self.update()
-            # self.axis.draw()
-            self.edit3.setText(f"{self.dispersion:.3f}")
+            self.spec_dim = -1
+            for dim, axis in self.dataset._axes.items():
+                if axis.dimension_type == sidpy.DimensionType.SPECTRAL:
+                    self.spec_dim = dim
+            if self.spec_dim < 0:
+                raise TypeError('We need at least one SPECTRAL dimension')
+
+            self.energy_scale = self.dataset._axes[self.spec_dim].values
+            self.dispersion = self.energy_scale[1] - self.energy_scale[0]
+            self.offset = self.energy_scale[0]
+            self.spectrum = np.zeros(2)
+
             self.change = 0
 
-    def on_close(self):
-        sender = self.sender()
-        if sender == self.ok_button:
-            pass
-            self.dataset.set_dimension(self.spec_dim, sidpy.Dimension(self.energy_scale, name='energy_scale',
-                                                                      units='eV', quantity='energy loss',
-                                                                      dimension_type='spectral'))
-        else:
-            pass
-        self.selector.set_visible(False)
-        self.signal_selected[bool].emit(True)
-        self.accept()
-        
-    def plot(self):
-        if self.dataset.data_type == sidpy.DataType.SPECTRAL_IMAGE:
-            self.spectrum = self.dataset.view.get_spectrum()
-        else:
-            self.spectrum = np.array(self.dataset)
-        x_limit = self.axis.get_xlim()
-        y_limit = self.axis.get_ylim()
+            self.x_min = self.energy_scale[int(len(self.energy_scale)/4)]
+            self.x_max = self.energy_scale[int(len(self.energy_scale) / 4*3)]
+            self.setWindowTitle("Select Energy")
 
-        self.axis.clear()
-        self.cplot = self.axis.plot(self.energy_scale, self.spectrum, label='spectrum')
-        self.axis.set_xlim(x_limit)
-        self.axis.set_ylim(y_limit)
+            valid_float = QtGui.QDoubleValidator()
 
-        self.axis.figure.canvas.draw()
+            layout = QtWidgets.QGridLayout()
+            layout.setVerticalSpacing(2)
+            self.label1 = QtWidgets.QLabel('Start:')
+            self.edit1 = QtWidgets.QLineEdit('0')
+            self.edit1.setValidator(valid_float)
+            self.unit1 = QtWidgets.QLabel('eV')
 
-    def update(self):
-        x_limit = self.axis.get_xlim()
-        y_limit = self.axis.get_ylim()
-        self.selector.extents = (self.x_min, self.x_max)
+            self.label2 = QtWidgets.QLabel('End:')
+            self.edit2 = QtWidgets.QLineEdit('0')
+            self.edit2.setValidator(valid_float)
+            self.unit2 = QtWidgets.QLabel('eV')
 
-        x_limit = np.array(x_limit) + self.change
+            self.label3 = QtWidgets.QLabel('Dispersion:')
+            self.edit3 = QtWidgets.QLineEdit('0')
+            self.edit3.setValidator(valid_float)
+            self.unit3 = QtWidgets.QLabel('eV')
 
-        self.cplot[0].set_data(self.energy_scale, self.spectrum)
-        self.axis.set_xlim(x_limit)
-        self.axis.set_ylim(y_limit)
-        self.axis.figure.canvas.draw()
+            self.edit1.editingFinished.connect(self.on_enter)
+            self.edit2.editingFinished.connect(self.on_enter)
+            self.edit3.editingFinished.connect(self.on_enter)
+
+            layout.addWidget(self.label1, 0, 0)
+            layout.addWidget(self.edit1, 0, 1)
+            layout.addWidget(self.unit1, 0, 2)
+
+            layout.addWidget(self.label2, 1, 0)
+            layout.addWidget(self.edit2, 1, 1)
+            layout.addWidget(self.unit2, 1, 2)
+
+            layout.addWidget(self.label3, 2, 0)
+            layout.addWidget(self.edit3, 2, 1)
+            layout.addWidget(self.unit3, 2, 2)
+
+            self.ok_button = QtWidgets.QPushButton('OK')
+            self.ok_button.clicked.connect(self.on_close)
+            self.cancel_button = QtWidgets.QPushButton('Cancel')
+            self.cancel_button.clicked.connect(self.on_close)
+
+            layout.addWidget(self.ok_button, 3, 0)
+            layout.addWidget(self.cancel_button, 3, 2)
+
+            self.setLayout(layout)
+            self.edit1.setFocus()
+            self.plot()
+
+            self.selector = SpanSelector(self.axis, self.line_select_callback,
+                                         direction="horizontal",
+                                         span_stays=True,
+                                         props=dict(facecolor='blue', alpha=0.2))
+            self.edit1.setText(f'{self.x_min:.3f}')
+            self.edit2.setText(f'{self.x_max:.3f}')
+            self.edit3.setText(f'{self.dispersion:.4f}')
+            self.update()
+
+        def line_select_callback(self, eclick, erelease):
+            y_min, y_max = self.axis.get_ylim()
+            self.x_min = self.selector.extents[0]
+            self.x_max = self.selector.extents[1]
+            # self.selector.extents = (self.x_min, self.x_max, y_min, y_max)
+
+            self.edit1.setText(f'{self.x_min:.3f}')
+            self.edit2.setText(f'{self.x_max:.3f}')
+
+        def on_enter(self):
+            sender = self.sender()
+
+            if sender == self.edit1:
+                value = float(str(sender.displayText()).strip())
+                if value == self.x_min:
+                    return
+                self.change = value - self.x_min
+                self.x_min += self.change
+                self.x_max += self.change
+                self.offset += self.change
+
+                self.edit1.setText(f"{self.x_min:.2f}")
+                self.edit2.setText(f"{self.x_max:.2f}")
+
+                self.energy_scale = np.arange(len(self.energy_scale)) * self.dispersion + self.offset
+
+                self.update()
+                # self.axis.draw()
+                # self.setWindowTitle(f'shift, {self.change}, {self.x_min}')
+
+            elif sender == self.edit2:
+                value = float(str(sender.displayText()).strip())
+                if value == self.x_max:
+                    return
+                start_channel = np.searchsorted(self.energy_scale, self.x_min)
+                end_channel = np.searchsorted(self.energy_scale, self.x_max)
+
+                self.x_max = value
+
+                if end_channel - start_channel != 0:
+                    self.dispersion = (self.x_max - self.x_min) / (end_channel - start_channel)
+                self.offset = self.x_min - start_channel * self.dispersion
+                self.edit2.setText(f"{self.x_max:.3f}")
+                self.edit3.setText(f"{self.dispersion:.4f}")
+                self.energy_scale = np.arange(len(self.energy_scale)) * self.dispersion + self.offset
+
+                self.update()
+                # self.axis.draw()
+                # self.setWindowTitle(f'range, {self.change}, {self.dispersion}')
+
+            elif sender == self.edit3:
+                value = float(str(sender.displayText()).strip())
+                if self.dispersion == value:
+                    return
+
+                start_channel = np.searchsorted(self.energy_scale, self.x_min)
+                end_channel = np.searchsorted(self.energy_scale, self.x_max)
+                self.dispersion = value
+                self.energy_scale = np.arange(len(self.energy_scale)) * self.dispersion + self.offset
+                self.x_min = self.energy_scale[start_channel]
+                self.x_max = self.energy_scale[end_channel]
+                self.update()
+                # self.axis.draw()
+                self.edit3.setText(f"{self.dispersion:.3f}")
+                self.change = 0
+
+        def on_close(self):
+            sender = self.sender()
+            if sender == self.ok_button:
+                pass
+                self.dataset.set_dimension(self.spec_dim, sidpy.Dimension(self.energy_scale, name='energy_scale',
+                                                                          units='eV', quantity='energy loss',
+                                                                          dimension_type='spectral'))
+            else:
+                pass
+            self.selector.set_visible(False)
+            self.signal_selected[bool].emit(True)
+            self.accept()
+
+        def plot(self):
+            if self.dataset.data_type == sidpy.DataType.SPECTRAL_IMAGE:
+                self.spectrum = self.dataset.view.get_spectrum()
+            else:
+                self.spectrum = np.array(self.dataset)
+            x_limit = self.axis.get_xlim()
+            y_limit = self.axis.get_ylim()
+
+            self.axis.clear()
+            self.cplot = self.axis.plot(self.energy_scale, self.spectrum, label='spectrum')
+            self.axis.set_xlim(x_limit)
+            self.axis.set_ylim(y_limit)
+
+            self.axis.figure.canvas.draw()
+
+        def update(self):
+            x_limit = self.axis.get_xlim()
+            y_limit = self.axis.get_ylim()
+            self.selector.extents = (self.x_min, self.x_max)
+
+            x_limit = np.array(x_limit) + self.change
+
+            self.cplot[0].set_data(self.energy_scale, self.spectrum)
+            self.axis.set_xlim(x_limit)
+            self.axis.set_ylim(y_limit)
+            self.axis.figure.canvas.draw()
 
 
 class RegionSelector(object):
